@@ -14,6 +14,10 @@
 #ifndef __BLAKE2_IMPL_H__
 #define __BLAKE2_IMPL_H__
 
+#if defined(_WIN32) || defined(WIN32)
+#include <windows.h>
+#endif
+
 #include <stddef.h>
 #include <stdint.h>
 #include <string.h>
@@ -25,8 +29,10 @@
 
 static inline uint32_t load32( const void *src )
 {
-#if defined(NATIVE_LITTLE_ENDIAN) && !defined(HAVE_ALIGNED_ACCESS_REQUIRED)
-  return *( uint32_t * )( src );
+#if defined(NATIVE_LITTLE_ENDIAN)
+  uint32_t w;
+  memcpy( &w, src, sizeof( w ) );
+  return w;
 #else
   const uint8_t *p = ( uint8_t * )src;
   uint32_t w = *p++;
@@ -39,8 +45,10 @@ static inline uint32_t load32( const void *src )
 
 static inline uint64_t load64( const void *src )
 {
-#if defined(NATIVE_LITTLE_ENDIAN) && !defined(HAVE_ALIGNED_ACCESS_REQUIRED)
-  return *( uint64_t * )( src );
+#if defined(NATIVE_LITTLE_ENDIAN)
+  uint64_t w;
+  memcpy( &w, src, sizeof( w ) );
+  return w;
 #else
   const uint8_t *p = ( uint8_t * )src;
   uint64_t w = *p++;
@@ -57,8 +65,8 @@ static inline uint64_t load64( const void *src )
 
 static inline void store32( void *dst, uint32_t w )
 {
-#if defined(NATIVE_LITTLE_ENDIAN) && !defined(HAVE_ALIGNED_ACCESS_REQUIRED)
-  *( uint32_t * )( dst ) = w;
+#if defined(NATIVE_LITTLE_ENDIAN)
+  memcpy( dst, &w, sizeof( w ) );
 #else
   uint8_t *p = ( uint8_t * )dst;
   *p++ = ( uint8_t )w; w >>= 8;
@@ -70,8 +78,8 @@ static inline void store32( void *dst, uint32_t w )
 
 static inline void store64( void *dst, uint64_t w )
 {
-#if defined(NATIVE_LITTLE_ENDIAN) && !defined(HAVE_ALIGNED_ACCESS_REQUIRED)
-  *( uint64_t * )( dst ) = w;
+#if defined(NATIVE_LITTLE_ENDIAN)
+  memcpy( dst, &w, sizeof( w ) );
 #else
   uint8_t *p = ( uint8_t * )dst;
   *p++ = ( uint8_t )w; w >>= 8;
@@ -131,8 +139,21 @@ static inline uint64_t rotr64( const uint64_t w, const unsigned c )
 /* prevents compiler optimizing out memset() */
 static inline void secure_zero_memory(void *v, size_t n)
 {
-  static void *(*const volatile memset_v)(void *, int, size_t) = &memset;
-  memset_v(v, 0, n);
+#if defined(_WIN32) || defined(WIN32)
+  SecureZeroMemory(v, n);
+#else
+// prioritize first the general C11 call
+#if defined(HAVE_MEMSET_S)
+  memset_s(v, n, 0, n);
+#elif defined(HAVE_EXPLICIT_BZERO)
+  explicit_bzero(v, n);
+#elif defined(HAVE_EXPLICIT_MEMSET)
+  explicit_memset(v, 0, n);
+#else
+  memset(v, 0, n);
+  __asm__ __volatile__("" :: "r"(v) : "memory");
+#endif
+#endif
 }
 
 #endif
